// VinciaAntenna.h is a part of the VINCIA plug-in to the PYTHIA 8 generator.
// Copyright (C) 2013 P. Skands
// VINCIA is licenced under the GNU GPL version 2.
// Please respect the MCnet Guidelines for Event Generator Authors and Users, 
//               http://www.montecarlonet.org/GUIDELINES

// This file contains header information for the Antenna class 
  
#ifndef VINCIA_VinciaAntenna_H
#define VINCIA_VinciaAntenna_H

#include "Basics.h"
#include "Event.h"
#include "PythiaStdlib.h"
#include "VinciaCommon.h"      
#include "VinciaFunctions.h"   // needed for DGLAP tests

namespace Vincia {
  
  // *****************************************************************
  // class Antenna :
  // Base class implementation for generic Antenna objects

  class Antenna {
    
  public:
    
    // Constructor
    Antenna() {
      // Init flags
      isInitPtr = false;
      isInit    = false;      
      // Default at construction time is global antenna (reset by init)
      isSectorSav          = false;
      // Default values (reset by init)
      verbose              = 1;
      useCollinearTermsSav = true;
      useFiniteTermsSav    = true;
      // Initialize LH and RH maps (DO NOT CHANGE)
      RH[9]  = true;
      RH[1]  = true;
      RH[-1] = false;
      LH[9]  = true;
      LH[1]  = false;
      LH[-1] = true;
      // At construction time, everything is zero (reset by init)
      hasFiniteTerms  = false;
      Cs00   = 0.0;
      Cs10   = 0.0;
      Cs01   = 0.0;
      Cf00   = 0.0;
      Cf10   = 0.0;
      Cf01   = 0.0;
      Cp00   = 0.0;
      Cp10   = 0.0;
      Cp01   = 0.0;
      chargeFactorSav = 0.0;
      kineMapTypeSav  = 0;
    };
    
    // Destructor 
    virtual ~Antenna() {};
    
    //=================================================================
    // List of VIRTUAL functions to be redefined by inheriting classes

    // Names of this antenna, for VINCIA, and for humans
    virtual string vinciaName()   const {return "Vincia:Antenna";};

    // Parton types (AB -> IJK): needed by soft- and collinear-limit checks 
    virtual int idA()    const {return 0;}
    virtual int idB()    const {return 0;}
    virtual int idJ()    const {return 0;}

    // The (dimensionless) antenna function.
    // Most generic form including masses and helicities.
    // Default parameter assignments => massless and unpolarized.
    virtual double antennaFunction(double yij, double yjk, 
		     double mui=0.0, double muj=0.0, double muk=0.0,
		     int hA=9, int hB=9, int hi=9, int hj=9, int hk=9);
    
    // Optional implementation of the AP kernels corresponding
    // to the two sides of the antenna, for collinear-limit checks
    // Defined as PI/yij + PK/yjk, i.e., equivalent to antennae.
    virtual double AltarelliParisi(double yij, double yjk,
				   double mui=0., double muj=0., double muk=0.,
                                   int hA=9, int hB=9, 
                                   int hI=9, int hJ=9, int hK=9);

    //=================================================================
    // List of VIRTUAL functions that MAY be redefined (if needed)

    // Default initialization (minMaxVar = 0 -> user settings)
    virtual bool init(bool isSectorIn = false, int minMaxVar = 0);
      
    // Construct baseName from idA, idB, and idJ
    virtual string baseName()    const {
      string name = id2str(idJ()) + "/" + id2str(idA()) + id2str(idB());
      return name;
    }

    // Wrapper that appends (sct) to name if antenna is sector type
    // Is made virtual so can in principle be redefined, but should
    // be fine for most implementations
    virtual string humanName()   const {
      string antName = baseName();
      if (isSectorSav) antName +=" (sct)";
      return antName;
    }

    // Function to check singularities, positivity, etc.
    virtual bool check();

    //=================================================================
    // List of NON-VIRTUAL functions (i.e., that may not be redefined)

    // Initialize pointers:
    //    ParticleData : required for masses, color charges, etc
    //    Rndm : required to generate random test points to check antenna
    //    Settings : required for all coefficients, switches, etc.
    void initPtr(Functions* functionsPtrIn, ParticleData* particleDataPtrIn, 
                 Rndm* rndmPtrIn, Settings* settingsPtrIn) {
      functionsPtr    = functionsPtrIn;
      particleDataPtr = particleDataPtrIn; 
      rndmPtr         = rndmPtrIn;
      settingsPtr     = settingsPtrIn; 
      isInitPtr       = true;
      isInit          = false;
    };

    // -------
    // Functions to GET

    // Shorthand to get the massless antenna function
    double antennaMassless(double yij, double yjk, int hA=9, int hB=9, 
			   int hi=9, int hj=9, int hk=9) {
      return antennaFunction(yij,yjk,0.0,0.0,0.0,hA,hB,hi,hj,hk);
    }

    // Get parameters
    bool isOn() { return isOnSav; }
    bool isSector() { return isSectorSav; };    
    bool useFiniteTerms() { return useFiniteTermsSav; }
    bool useCollinearTerms() { return useCollinearTermsSav; }
    bool useSectorTerms() {
      if (isSectorSav) return useSectorTermsSav;
      else return false;
    }
    double getChargeFactor() { return chargeFactorSav; }
    int    getKineMapType() { return kineMapTypeSav; }
    double alpha() { return alphaSav; }

    // Shorthand for temporarily switching off finite terms to check
    // singular part only
    double getSingularTerms(double yij, double yjk, 
			    double mui=0.0, double muj=0.0, double muk=0.0,
			    int hA=9, int hB=9, 
			    int hi=9, int hj=9, int hk=9) {
      // Temporarily switch off finite terms 
      bool useFiniteTermsOrg = useFiniteTermsSav;
      useFiniteTermsSav = false;
      double a = antennaFunction(yij,yjk,mui,muj,muk,hA,hB,hi,hj,hk);
      // Restore finite terms and return
      useFiniteTermsSav = useFiniteTermsOrg;
      return a;
    }

    // Functions to get Altarelli-Parisi energy fractions from scaled invariants
    double zi(double yij,double yjk) {return (1.-yjk)/(1.+yij);}
    double zk(double yij,double yjk) {return (1.-yij)/(1.+yjk);}

    // *****************************************************************
    // id2str()
    // Auxiliary function to translate an ID code to a string, 
    // for constructing antenna names
    
    string id2str(int id) const {
      // Vector bosons
      if (id == 21) return "g";
      else if (id == 22) return "gamma";
      else if (id == 23) return "Z";
      else if (abs(id) == 24) return "W";
      // Fermions
      else if (id >= 1 && id <= 10) return "q";
      else if (-id >= 1 && -id <= 10) return "qbar";
      else if (id >= 11 && id <= 20 && id%2 == 1) return "l-"; 
      else if (id >= 11 && id <= 20 && id%2 == 0) return "nu"; 
      else if (-id >= 11 && -id <= 20 && id%2 == 1) return "l+"; 
      else if (-id >= 11 && -id <= 20 && id%2 == 0) return "nubar"; 
      // Octet fermion: use gluino
      else if (id == 1000021) return "~g";
      // Charged scalar: use H+
      else if (id == 37) return "H+";
      else if (id == -37) return "H-";
      // Colored scalars: use squarks
      else if (id >= 1000000 && id <= 1000010) return "~q";
      else if (-id >= 1000000 && -id <= 1000010) return "~q*";
      // Unknown particle
      else return "X";
    }
  
    //=================================================================
    // List of PROTECTED data members
    // (Not private, so can be accessed by inheriting classes)

  protected:
    
    // Is initialized
    bool isInitPtr, isInit;
    
    // Type of antenna
    bool isSectorSav;    

    // Antenna and terms on/off
    bool isOnSav;
    bool useFiniteTermsSav;
    bool useCollinearTermsSav;
    bool useSectorTermsSav;
    
    // Charge factor and kinematics map
    double chargeFactorSav;
    int kineMapTypeSav;

    // The alpha collinear-partitioning parameter
    double alphaSav;
    
    // Finite terms: separate for s-wave (MHV), s-wave (NMHV), and p-wave
    bool hasFiniteTerms;
    double Cs00, Cs01, Cs10;
    double Cf00, Cf01, Cf10;
    double Cp00, Cp01, Cp10;
    // So far not allowing finite terms for sector antennae with external
    // helicity flips, but these could in principle also be added here.    
    
    // Map to tell whether a given helicity value maps to L- and/or R-handed
    // Defined by constructor and not to be changed dynamically.
    map<int,bool> LH;
    map<int,bool> RH;

    // Verbose level
    int verbose;

    // Pointers to Vincia classes
    Functions* functionsPtr;

    // Pointers to Pythia-8 classes
    ParticleData* particleDataPtr;
    Rndm*         rndmPtr;
    Settings*     settingsPtr;  
    
  };

  //**************************************************************************
  //
  // Inherited Antennae

  //--------------------------------------------------------------------------
  // class QQEmit 

  class QQEmit : public Antenna {

    // Names (remember to redefine both for each inherited class)
    virtual string vinciaName()    const {return "Vincia:QQemit";};

    // Functions needed by soft- and collinear-limit checks (AB -> IJK)
    virtual int idA()    const {return 1;}
    virtual int idB()    const {return -1;}
    virtual int idJ()    const {return 21;}

    // Init
    virtual bool init(bool isSectorIn = false, int minMaxVar = 0);

    // The (dimensionless) antenna function
    virtual double antennaFunction(double yij, double yjk, 
		     double mui=0.0, double muj=0.0, double muk=0.0,
		     int hA=9, int hB=9, int hi=9, int hj=9, int hk=9);

    // Auxiliary: unpolarized antenna function for massive particles
    virtual double antennaMassive(double yij, double yjk, 
				  double mui, double muk);
    
    // Function to give Altarelli-Parisi limits of this antenna
    // Defined as PI/yij + PK/yjk, i.e., equivalent to antennae.
    virtual double AltarelliParisi(double yij, double yjk, 
                                   double, double, double, int hA=9, int hB=9, 
                                   int hI=9, int hJ=9, int hK=9){
      if (hA != hI || hB != hK) return -1.;
      else return functionsPtr->Pq2qg(zi(yij,yjk),hA,hI,hJ)/yij 
             + functionsPtr->Pq2qg(zk(yij,yjk),hB,hK,hJ)/yjk;
    }

    // Additional data members specific to this class
  private:

    bool hasLinearM, hasSquareM, hasCrossM;
    double M1000, M1010, M2000, M2010, M1100, M1110;
    double denominatorMassPrefactor;

  };


  //--------------------------------------------------------------------------
  // class QGEmit 

  class QGEmit : public Antenna {

    // Names (remember to redefine both for each inherited class)
    virtual string vinciaName()    const {return "Vincia:QGemit";};

    // Parton types (AB -> IJK): needed by soft- and collinear-limit checks 
    virtual int idA()    const {return 1;}
    virtual int idB()    const {return 21;}
    virtual int idJ()    const {return 21;}

    // Init
    virtual bool init(bool isSectorIn = false, int minMaxVar = 0);

    // The (dimensionless) antenna function
    virtual double antennaFunction(double yij, double yjk, 
		     double mui=0.0, double muj=0.0, double muk=0.0,
		     int hA=9, int hB=9, int hi=9, int hj=9, int hk=9);

    // Auxiliary: unpolarized antenna function for massive particles
    virtual double antennaMassive(double yij, double yjk, double mui);
    
    // Function to give Altarelli-Parisi limits of this antenna
    virtual double AltarelliParisi(double yij, double yjk,
                                   double, double, double, int hA=9, int hB=9, 
                                   int hI=9, int hJ=9, int hK=9) {
      double sum(0.);
      if (hI != hA) return -1;
      if (hK == hB) sum += functionsPtr->Pq2qg(zi(yij,yjk),hA,hI,hJ)/yij;
      sum += functionsPtr->Pg2gg(zk(yij,yjk),hB,hK,hJ)/yjk;
      return sum;
    }

    // Additional data members specific to this class
  private:

    bool hasLinearM, hasSquareM, hasTildeM;
    double M100, M110, M101, M200, M210, M201, M2m0, M2m1;
    double T2mm2, T3mm1, T4mm1;

  };

  //--------------------------------------------------------------------------
  // class GGEmit 

  class GGEmit : public Antenna {

    // Names (remember to redefine both for each inherited class)
    virtual string vinciaName()    const {return "Vincia:GGemit";};

    // Parton types (AB -> IJK): needed by soft- and collinear-limit checks 
    virtual int idA()    const {return 21;}
    virtual int idB()    const {return 21;}
    virtual int idJ()    const {return 21;}

    // Init
    virtual bool init(bool isSectorIn = false, int minMaxVar = 0);

    // The (dimensionless) antenna function
    virtual double antennaFunction(double yij, double yjk, 
		     double mui=0.0, double muj=0.0, double muk=0.0,
		     int hA=9, int hB=9, int hi=9, int hj=9, int hk=9);

    // Function to give Altarelli-Parisi limits of this antenna
    virtual double AltarelliParisi(double yij, double yjk,
                                   double, double, double, int hA=9, int hB=9, 
                                   int hI=9, int hJ=9, int hK=9) {
      double sum(0.);
      if ( hB == hK ) sum += functionsPtr->Pg2gg(zi(yij,yjk),hA,hI,hJ)/yij;
      if ( hA == hI ) sum += functionsPtr->Pg2gg(zk(yij,yjk),hB,hK,hJ)/yjk;
      return sum;
    }

  };

  //--------------------------------------------------------------------------
  // class QGSplit

  class QGSplit : public Antenna {

    // Names (remember to redefine both for each inherited class)
    virtual string vinciaName()    const {return "Vincia:QGsplit";};

    // Parton types (AB -> IJK): needed by soft- and collinear-limit checks 
    virtual int idA()    const {return 1;}
    virtual int idB()    const {return 21;}
    virtual int idJ()    const {return -2;}

    // Init
    virtual bool init(bool isSectorIn = false, int MinMaxVar = 0);

    // The (dimensionless) antenna function
    virtual double antennaFunction(double yij, double yjk, 
		     double mui=0.0, double muj=0.0, double muk=0.0,
		     int hA=9, int hB=9, int hi=9, int hj=9, int hk=9);

    // Auxiliary: unpolarized antenna function for massive particles
    virtual double antennaMassive(double yij, double yjk, 
				  double mui, double muj);

    // Function to give Altarelli-Parisi limits of this antenna
    virtual double AltarelliParisi(double yij, double yjk,
                                   double, double, double, int hA=9, int hB=9, 
                                   int hI=9, int hJ=9, int hK=9) {
      if (hA != hI) return 0.0; 
      else return functionsPtr->Pg2qq(zk(yij,yjk),hB,hK,hJ)/yjk;
    }

    // Additional data members specific to this class
  private:

    bool hasLinearM, hasSquareM, hasTildeM, hasSquareD, hasQuarticD;
    double M1000, M1010, M1001, M2000, M2010, M2001;
    double T00, T10, T01;
    double T1000, T1010, T1001, T2000, T2010, T2001;
    double T0200, T0210, T0201;

  };

  //--------------------------------------------------------------------------
  // class GGSplit 
  // Note: function coded is for g g -> g qbar q
  // Vincia automatically includes the other permutation

  class GGSplit : public Antenna {

    // Names (remember to redefine both for each inherited class)
    virtual string vinciaName()    const {return "Vincia:GGsplit";};

    // Parton types (AB -> IJK): needed by soft- and collinear-limit checks 
    virtual int idA()    const {return 21;}
    virtual int idB()    const {return 21;}
    virtual int idJ()    const {return -2;}

    // Init
    virtual bool init(bool isSectorIn = false, int MinMaxVar = 0);

    // The (dimensionless) antenna function
    virtual double antennaFunction(double yij, double yjk, 
		     double mui=0.0, double muj=0.0, double muk=0.0,
		     int hA=9, int hB=9, int hi=9, int hj=9, int hk=9);

    // Auxiliary: unpolarized antenna function for massive particles
    virtual double antennaMassive(double yij, double yjk, double muj);    

    // Function to give Altarelli-Parisi limits of this antenna
    virtual double AltarelliParisi(double yij, double yjk,
                                   double, double, double, int hA=9, int hB=9, 
                                   int hI=9, int hJ=9, int hK=9) {
      if (hA != hI) return 0.;
      else return functionsPtr->Pg2qq(zk(yij,yjk),hB,hK,hJ)/yjk;
    }

    // Additional data members specific to this class
 private:

    bool hasSquareD, hasQuarticD;
    double T00, T10, T01;
    double M200, M210, M201;

  };
  
} // End Vincia namespace

#endif
