// VinciaSubtractionTerms.h is a part of the VINCIA plug-in to the PYTHIA 8 generator.
// Copyright (C) 2013 P. Skands.
// VINCIA is licenced under the GNU GPL version 2.
// Please respect the MCnet Guidelines for Event Generator Authors and Users, 
//               http://www.montecarlonet.org/GUIDELINES

// This file contains header information for VINCIA's Matching class.

#ifndef VINCIASUBTRACTIONTERMS_H
#define VINCIASUBTRACTIONTERMS_H

#include "VinciaCommon.h"   // Common Vincia functions
#include "math.h"

namespace Vincia {
  
  // *****************************************************************
  // class VinciaSubtractionTerms :
  // Base class implementation for generic subtraction terms 
  // Note: the base class implements the GGG (sub-)antennae;
  // see Gehrmann, Gehrmann-de Ridder, Glover: JHEP 0509 (2005) 056

  class SubtractionTerms {

  public:
      
    // Constructor.
    
    SubtractionTerms() {isInit=false;}

    // Destructor.

    ~SubtractionTerms() {}

    // Initialize Pointers (must be done before init)
    void initPtr(Settings* settingsPtrIn) {
      settingsPtr    = settingsPtrIn; 
      isInitPtr=true;
    }
    
    // Initialize
    bool init();

    // Differential subtraction terms (= sub-antennae)
    double AqqEmit(double yij,double yjk);
    double AqgEmit(double yij,double yjk);
    double AqgSplit(double yij,double yjk);
    // Wrapper to get an enumerated antenna
    double getA(int iAnt, double yij, double yjk) {
      if (iAnt == iQQemit) return AqqEmit(yij,yjk);
      if (iAnt == iQGemit) return AqgEmit(yij,yjk);
      if (iAnt == iQGsplit) return AqgSplit(yij,yjk);
      else return 0.;
    }
    
    // Integrated subtraction terms (over resolved region defined by
    // the two dimensionless invariants and the type of evolution variable)
    double IqqEmit(double yij,double yjk,int evType);
    double IqgEmit(double yij,double yjk,int evType,int ordType);
    double IqgSplit(double yij,double yjk,int evType,int ordType,double N);


  private:

    // Is initialized?    
    bool isInitPtr, isInit;
    
    // Pointer to PYTHIA 8 Settings database
    Settings* settingsPtr;

    // Verbose level
    int verbose;   

    // Declarations for strong pT ordering
    double yEpT(double,double);
    double DLStpT(double);
    //double SL1StpT(double);
    //double SL2StpT(double);
    double SL12StpT(double);
    double SL3StpT(double);
    double CStpT(double,double,double,double); 
    // Declarations for strong mD ordering
    double DLStmD(double);
    double SL1StmD(double);
    double SL2StmD(double);
    double SL3StmD(double);
    double CStmD(double,double,double,double);
    // Declarations for energy ordering
    static const double del;
    double yEEn(double,double);
    double DLEn(double);
    double SL1En(double);
    double SL2En(double);
    // Declarations for pT smooth ordering
    double DLSmpT(double);
    double SL1SmpT(double);
    double SL2SmpT(double);
    double SL3SmpT(double);
    double CSmpT(double,double,double,double);
    // Declarations for smooth mD ordering
    double DLSmmD(double);
    double SL1SmmD(double);
    double SL2SmmD(double);
    double SL3SmmD(double);
    double CSmmD(double,double,double,double); 
    // Declaration for mqq strong ordering
    double SL1Stmqq(double);
    double SL2Stmqq(double);
    double SL3Stmqq(double);
    double CStmqq(double,double,double,double); 
    // Declaration for mqq smooth pT ordering
    //static const double NpT;
    double SL1SmpTmqq(double,double);
    double SL2SmpTmqq(double,double);
    double SL3SmpTmqq(double,double);
    double CSmpTmqq(double,double,double,double,double);
    // Declaration for mqq smooth mD ordering
    //static const double NmD;
    double SL1SmmDmqq(double,double);
    double SL2SmmDmqq(double,double);
    double SL3SmmDmqq(double,double);
    double CSmmDmqq(double,double,double,double,double); 
 
  };

}

#endif
