// vincia05-root is a part of the VINCIA plugin to PYTHIA 8
// This is a simple test program to run Z->bbbar at LEP 1, 
// interfaced to ROOT, including a runtime updated display,
// using VINCIA's ROOT-based runtime display. 
// Uses VINCIA for showering and matching and PYTHIA 8 for all else
// Copyright (c) 2013 Peter Skands

// PYTHIA and VINCIA include files
#include "Vincia.h"
// ROOT: include files for linking to ROOT, uncomment as necessary
#include "VinciaRoot.h"
// FASTJET
// #include "fastjet/PseudoJet.hh"
// #include "fastjet/ClusterSequence.hh"

// Use Pythia8 and Vincia namespaces, for convenience.
using namespace Pythia8; 
using namespace Vincia; 

// Main Program
int main() {

  //************************************************************************
  // PYTHIA and VINCIA objects
  //************************************************************************

  // Create Pythia and Vincia objects
  Pythia pythia; 
  VinciaPlugin vincia(&pythia,"vincia05.cmnd");

  // Global flag whether to exclude neutrinos from clusterings, E_vis, etc
  bool excludeNeutrinos = false;

  //************************************************************************
  // Define some useful pointers and shorthands for later on
  //************************************************************************

  // Shorthand for event record
  Event& event = pythia.event;
  Settings* settingsPtr = vincia.settingsPtr;        // to settings database

  // Extract settings to be used in the main program.  
  double eCM       = settingsPtr->parm("Beams:eCM");
  int    nEvent    = settingsPtr->mode("Main:numberOfEvents");
  int    nShow     = settingsPtr->mode("Main:timesToShow");
  bool   vinciaOn  = settingsPtr->flag("Vincia");
  bool   hadronize = settingsPtr->flag("HadronLevel:all");
  int    verbose   = settingsPtr->mode("Vincia:verbose");

  // Extrac matching settings
  int    maxMatchTree     = settingsPtr->mode("Vincia:matchingOrder");
  bool   matching         = maxMatchTree >= 1;

  //************************************************************************
  // VinciaRoot
  //************************************************************************

  // Create VinciaRoot objects
  VinciaRoot vroot; 

  // Tell VinciaRoot which verbose level to use
  vroot.setVerbose(verbose);
  // Switch stats on/off (in ROOT conventions, but with 0 = off)
  vroot.setOptStat(0);
  // Set MC names
  string pyVer = num2str(settingsPtr->parm("Pythia:versionNumber"),5);  
  string vcVer = num2str(settingsPtr->parm("Vincia:versionNumber"),5);  
  string MCnames;
  if (vinciaOn) {
    MCnames += "Vincia "+vcVer+" +";
    if (matching) MCnames += " MadGraph 4.426 + ";
  }
  MCnames += " Pythia "+pyVer;
  vroot.setMCnames(MCnames);

  // Set label for plot legend
  string mcLabel = "Vincia";  
  if (!vinciaOn) mcLabel = "Pythia";
  if (!hadronize) mcLabel += " (p)";
  vroot.setLegend(mcLabel);

  //************************************************************************
  // Initialize PYTHIA (automatically initializes VINCIA) 
  //************************************************************************

  // Initialize
  pythia.init();

  // List changed settings
  pythia.settings.listChanged();

  // Check that Z0 decay channels set correctly.
  pythia.particleData.listChanged();
 
  //***************************************************************************
  // ROOT
  //***************************************************************************

  // Form name for ROOT file
  ostringstream outfile;
  // Put root files in subdirectory defined at beginning of file
  if (not vinciaOn) {
    outfile<<"pythia";
  } else {
    outfile<<"vincia";
  }
  if (settingsPtr->flag("HadronLevel:all")) outfile<<"-had";
  outfile<<".root";

  //***************************************************************************
  // Book Histograms
  vroot.setLogY();
  
  // Vincia uncertainty bands: number of variations expected 
  int nVariations = max(1,vincia.nVariations());

  // Thrust
  vector<TH1D*> HThrust;
  HThrust.push_back(new TH1D("Thrust","1-T (b)", 100, 0.0, 0.4));
  double wThrust = 100.0/0.4;
  HThrust[0]->GetXaxis()->SetTitle("1-T (b)");
  HThrust[0]->GetYaxis()->SetTitle("1/N dN/d(1-T)");
  HThrust[0]->SetMinimum(0.0005);
  HThrust[0]->SetMaximum(100.);
  for (int iVar=1; iVar<nVariations; iVar++) 
    HThrust.push_back(static_cast<TH1D*>(
                HThrust[0]->Clone(string("Thrust"+num2str(iVar,1)).c_str())));
  vroot.createDisplay("T",HThrust,350,400);
  vroot.setDataFile("T","data/L3-91-Thrust-b.vec",12,32,40,51,52,61,62);
  
  // C
  vector<TH1D*> HC;
  HC.push_back(new TH1D("C","C parameter (b)", 40, 0.0, 1.0));
  double wC = 40.0/1.0;
  HC[0]->GetXaxis()->SetTitle("C (b)");
  HC[0]->GetYaxis()->SetTitle("1/N dN/dC");
  HC[0]->SetMinimum(0.0005);
  HC[0]->SetMaximum(100.);
  for (int iVar=1; iVar<nVariations; iVar++) 
    HC.push_back(static_cast<TH1D*>(
                          HC[0]->Clone(string("C"+num2str(iVar,1)).c_str())));
  vroot.createDisplay("C",HC,350,400);
  vroot.setDataFile("C","data/L3-91-C-b.vec",10,30,40,51,52,61,62);
  
  // D (4-jet obs)
  vector<TH1D*> HD;
  HD.push_back(new TH1D("D","D parameter (b)", 50, 0.0, 0.8));
  double wD = 50.0/0.8;
  HD[0]->GetXaxis()->SetTitle("D (b)");
  HD[0]->GetYaxis()->SetTitle("1/N dN/dD");
  HD[0]->SetMinimum(0.0005);
  HD[0]->SetMaximum(100.);
  for (int iVar=1; iVar<nVariations; iVar++) 
    HD.push_back(static_cast<TH1D*>(
                          HD[0]->Clone(string("D"+num2str(iVar,1)).c_str())));
  vroot.createDisplay("D",HD,350,400);
  vroot.setDataFile("D","data/L3-91-D-b.vec",10,30,40,51,52,61,62);
  
  // Heavy-Jet Mass (3-jet obs)
  vector<TH1D*> HMjH;
  HMjH.push_back(new TH1D("HeavyJetMass","Heavy Jet Mass (b)",40, 0.0, 0.3));
  double wMjH = 40.0/0.3;
  HMjH[0]->SetMaximum(100.01); 
  HMjH[0]->SetMinimum(0.005);
  for (int iVar=1; iVar<nVariations; iVar++) 
    HMjH.push_back(static_cast<TH1D*>(
              HMjH[0]->Clone(string("HeavyJetMass"+num2str(iVar,1)).c_str())));
  vroot.createDisplay("MH",HMjH,350,400);
  vroot.setDataFile("MH","data/L3-91-MH-b.vec",10,30,40,51,52,61,62);
  
  // Wide Jet Broadening
  vector<TH1D*> HBjWide;
  HBjWide.push_back(
        new TH1D("WideJetBroadening","Wide Jet Broadening (b)",40, 0.0, 0.3));
  double wBjWide = 40.0/0.3;
  HBjWide[0]->GetXaxis()->SetTitle("B_{W} (b)");
  HBjWide[0]->GetYaxis()->SetTitle("1/N dN/dB_{W}");
  HBjWide[0]->SetMaximum(100.01); 
  HBjWide[0]->SetMinimum(0.005);
  for (int iVar=1; iVar<nVariations; iVar++) 
    HBjWide.push_back(static_cast<TH1D*>(
      HBjWide[0]->Clone(string("WideJetBroadening"+num2str(iVar,1)).c_str())));
  vroot.createDisplay("BW",HBjWide,350,400);
  vroot.setDataFile("BW","data/L3-91-BWide-b.vec",10,30,40,51,52,61,62);

  // Total Jet Broadening
  vector<TH1D*> HBjTot;
  HBjTot.push_back(
      new TH1D("TotalJetBroadening","Total Jet Broadening (b)",40, 0.0, 0.4));
  double wBjTot = 40.0/0.4;
  HBjTot[0]->GetXaxis()->SetTitle("B_{T} (b)");
  HBjTot[0]->GetYaxis()->SetTitle("1/N dN/dB_{T}");
  HBjTot[0]->SetMaximum(100.01); 
  HBjTot[0]->SetMinimum(0.005);
  for (int iVar=1; iVar<nVariations; iVar++) 
    HBjTot.push_back(static_cast<TH1D*>(
      HBjTot[0]->Clone(string("TotalJetBroadening"+num2str(iVar,1)).c_str())));
  vroot.createDisplay("BT",HBjTot,350,400);
  vroot.setDataFile("BT","data/L3-91-BTot-b.vec",10,30,40,51,52,61,62);

  // nCharged
  vector<TH1D*> HnCharged;
  HnCharged.push_back(new TH1D("nCharged","nCharged (b)", 60, -0.5, 59.5));
  HnCharged[0]->GetXaxis()->SetTitle("N_{ch} (b)");
  HnCharged[0]->GetYaxis()->SetTitle("Probability(N_{ch})");
  HnCharged[0]->SetMinimum(0.00002);
  HnCharged[0]->SetMaximum(0.5);
  for (int iVar=1; iVar<nVariations; iVar++) 
    HnCharged.push_back(static_cast<TH1D*>(
            HnCharged[0]->Clone(string("nCharged"+num2str(iVar,1)).c_str())));
  vroot.createDisplay("Nch",HnCharged,350,400);
  vroot.setDataFile("Nch","data/L3-91-Nch-b.vec",20,40,50,60);

  // Ln(1/x)
  vector<TH1D*> Hx;
  Hx.push_back(new TH1D("LnXInv","Ln(1/x) (b)", 200, 0.0, 8.0));
  double wx = 200.0/8.0;
  Hx[0]->GetXaxis()->SetTitle("Ln(1/x) (b)");
  Hx[0]->GetYaxis()->SetTitle("1/N_{ch} dN_{ch}/dLn(1/x)");
  Hx[0]->SetMinimum(0.0002);
  Hx[0]->SetMaximum(5.);
  for (int iVar=1; iVar<nVariations; iVar++) 
    Hx.push_back(static_cast<TH1D*>(
                      Hx[0]->Clone(string("LnXInv"+num2str(iVar,1)).c_str())));
  vroot.createDisplay("x",Hx,350,400);
  vroot.setDataFile("x","data/L3-91-LogInvx-b.vec",10,30,40,51,52,61,62);

  // xB
  vector<TH1D*> HxBSLD;
  HxBSLD.push_back(new TH1D("XBSLD","x_{B}^{weak} (SLD)", 50, 0.0, 1.0));
  double wxBSLD = 50.0/1.0;
  HxBSLD[0]->GetXaxis()->SetTitle("x");
  HxBSLD[0]->GetYaxis()->SetTitle("1/N dN/dx_{B}");
  HxBSLD[0]->SetMinimum(0.02);
  HxBSLD[0]->SetMaximum(20.);
  for (int iVar=1; iVar<nVariations; iVar++) 
    HxBSLD.push_back(static_cast<TH1D*>(
                  HxBSLD[0]->Clone(string("XBSLD"+num2str(iVar,1)).c_str())));
  vroot.createDisplay("xB-SLD",HxBSLD,350,400);
  vroot.setDataFile("xB-SLD","data/SLD-91-xBweak.vec",20,10,30,40,51,52,61,62,71,72);

  vector<TH1D*> HxBDelphi;
  HxBDelphi.push_back(
                  new TH1D("XBDelphi","x_{B}^{weak} (DELPHI)", 50, 0.0, 1.0));
  double wxBDelphi = 50.0/1.0;
  HxBDelphi[0]->GetXaxis()->SetTitle("x");
  HxBDelphi[0]->GetYaxis()->SetTitle("1/N dN/dx_{B}");
  HxBDelphi[0]->SetMinimum(0.02);
  HxBDelphi[0]->SetMaximum(20.);
  for (int iVar=1; iVar<nVariations; iVar++) 
    HxBDelphi.push_back(static_cast<TH1D*>(
            HxBDelphi[0]->Clone(string("XBDelphi"+num2str(iVar,1)).c_str())));
  vroot.createDisplay("xB-Delphi",HxBDelphi,350,400);
  vroot.setDataFile("xB-Delphi","data/Delphi-91-xBweak.vec",10,30,40,50,60,70);

  // Event analysis tools with or without neutrinos included
  int select = 1;
  if (excludeNeutrinos) select = 2;
  Thrust Thrust(select);
  Sphericity SphLin(1,select),SphSq(2,select);
  ClusterJet ktClu("Durham",select,2);

  //***************************************************************************
  // EVENT GENERATION LOOP
  // Generation, event-by-event printout, analysis, and storage
  //***************************************************************************

  // (Optionally) re-initialize random number engine before event loop
  const int initseed = settingsPtr->mode("Random:seed");
  vincia.rndmPtr->init(initseed);

  // Begin event loop
  double nHasB = 0.0;
  for (int iEvent = 0; iEvent < nEvent; ++iEvent) {

    if (iEvent%(max(1,nEvent/max(nShow,1))) == 0)  
      cout << " Now begin event " << iEvent << endl;

    if (!pythia.next()) continue;

    // Count FS charged hadrons, partons, and quarks
    int nc=0;
    int np=0;
    int nq=0;

    // Check for presence of b quark and compute visible energy 
    Vec4 pVis(0.0);
    bool hasB = false;
    for (int i=1;i<event.size();i++) {

      if (pythia.event[i].id() == 5) hasB = true;

      // Final state only
      if (!pythia.event[i].isFinal())        continue;
      
      // No neutrinos
      if ( excludeNeutrinos ) 
        if (pythia.event[i].idAbs() == 12 || pythia.event[i].idAbs() == 14 ||
            pythia.event[i].idAbs() == 16)     continue;
      
      pVis += event[i].p();
    }
    // Skip event if no b quarks
    if ( !hasB ) continue;
    else ++nHasB;

    // Energy component
    double eVis = pVis.e();
                                                
    for (int i=1;i<event.size();i++) {
      // Count up final-state charged hadrons
      if (event[i].isCharged() && event[i].isFinal()) {
        nc++;
        double LnXInv=log(double(1.0/(2.0*event[i].pAbs()/eCM)));
        for (int iVar=0; iVar<nVariations; iVar++) {
          Hx[iVar]->Fill(LnXInv, wx*vincia.weight(iVar));
        }
      }
      // Find last parton-level partons
      int iDau1 = event[i].daughter1();
      if (iDau1 == 0 || abs(event[iDau1].status()) > 80) {
        // Count up partons and quarks
        if (event[i].isQuark() || event[i].isGluon() ) np++;
        if (event[i].isQuark()) nq++;        
      }
      double x = 2.*event[i].pAbs() / eCM;

      // Find weakly decaying B mesons 
      int idp = abs(event[i].id());
      double xBweak = -1.0;
      if (hadronize) {
        if (idp > 510 && idp < 600) {
          if (abs(event[event[i].daughter1()].id()) < 500) {
            xBweak = x;
          }
        }
        // Find weakly decaying B baryons
        if (idp > 5100 && idp < 6000) {
          if (abs(event[event[i].daughter1()].id()) < 5000) {
            xBweak = x;
          }        
        }
      } 
      // If hadronization switched off, just plot x of b quarks
      else {
        if (idp == 5 && event[i].isFinal()) xBweak = x;
      }
      if (xBweak > 0.) {
        for (int iVar=0; iVar<nVariations; iVar++) {
          HxBSLD[iVar]->Fill(xBweak, wxBSLD*vincia.weight(iVar));
          HxBDelphi[iVar]->Fill(xBweak, wxBDelphi*vincia.weight(iVar));
        }
      }        
    }
    
    // Fill charged-particle histogram
    for (int iVar=0; iVar<nVariations; iVar++) {
      HnCharged[iVar]->Fill(1.0*nc, vincia.weight(iVar));
    }
    
    // Event Shape analysis
    Thrust.analyze( event );
    for (int iVar=0; iVar < nVariations; iVar++) {
      HThrust[iVar]->Fill(1-Thrust.thrust(),wThrust*vincia.weight(iVar));
    }

    // Histogram Linear Sphericity values
    if (np >= 2.0) {
      SphLin.analyze( event );
      SphSq.analyze( event );
      double evC=3*(SphLin.eigenValue(1)*SphLin.eigenValue(2)
                    + SphLin.eigenValue(2)*SphLin.eigenValue(3)
                    + SphLin.eigenValue(3)*SphLin.eigenValue(1));
      double evD=27*SphLin.eigenValue(1)*SphLin.eigenValue(2)
        *SphLin.eigenValue(3);
      for (int iVar=0; iVar < nVariations; iVar++) {
        HC[iVar]->Fill(evC,wC*vincia.weight(iVar));
        HD[iVar]->Fill(evD,wD*vincia.weight(iVar));
      }
      
    }


    // Histogram jet masses
    Vec4 thrustAxis = Thrust.eventAxis(1);   
    Vec4 pPlus, pMinus;
    double bPlus(0.), bMinus(0.), pPlusAbs(0.), pMinusAbs(0.);
    // Loop over event to compute masses
    for (int i=0; i<event.size(); i++) {
      if (not event[i].isFinal() ) continue;
      int id = event[i].id();
      if ( excludeNeutrinos )
        if (abs(id) == 12 || abs(id) == 14 || abs(id) == 16) continue;
      double dotp = event[i].px()*thrustAxis.px()
        + event[i].py()*thrustAxis.py()
        + event[i].pz()*thrustAxis.pz();
      double pAbs = event[i].pAbs();
      double cosT = dotp/pAbs/thrustAxis.pAbs();
      double sinT = sqrt(1.-pow2(cosT));
      if (dotp > 0) {
        pPlus     += event[i].p();
        bPlus     += abs(pAbs * sinT);
        pPlusAbs  += pAbs;
      } else {
        pMinus += event[i].p();
        bMinus    += abs(pAbs * sinT);
        pMinusAbs += pAbs;
      }
    }
    bPlus  /= 2.*(pPlusAbs+pMinusAbs);
    bMinus /= 2.*(pMinusAbs+pPlusAbs);    
    double mPlus  = pPlus.mCalc();
    double mMinus = pMinus.mCalc();
    for (int iVar=0; iVar < nVariations; iVar++) {
      HMjH[iVar]->Fill(pow2(max(mPlus,mMinus)/eVis),wMjH*vincia.weight(iVar));
      HBjWide[iVar]->Fill(max(bPlus,bMinus),wBjWide*vincia.weight(iVar));
      HBjTot[iVar]->Fill(bPlus+bMinus,wBjTot*vincia.weight(iVar)); 
    }  
    
    // Periodically update displays
    if (int(nHasB) == 9 || int(nHasB) == 99 || 
        int(nHasB)%500 == 99|| iEvent == nEvent-1) {
      vroot.updateDisplay();       
      if ( (nHasB >= 1000 && int(nHasB)%5000==99) || iEvent == nEvent-1) {
        string mcStr="vc";
        if (not vinciaOn) mcStr="py";
        else {
          mcStr += "-m"+num2str(maxMatchTree,1);
        }
        vroot.saveDisplays(mcStr, "pdf");

        // Save Pythia histograms for overplotting later
        if (! vinciaOn) vroot.saveHistograms("py05","dat");

      }
    }

  }
  
  // Print out Vincia internal histograms
  if (vinciaOn && verbose >= 2) vincia.printHistos();

  // Print out Vincia end-of-run information
  pythia.statistics();
  if (vinciaOn) vincia.printInfo();

  // Save histograms in ROOT file
  cout<<"Saving root file... "<<outfile.str()<<endl;
  // Open ROOT file
  TFile f(outfile.str().c_str(),"recreate");
  for (int iVar=0; iVar<nVariations; iVar++) {
    HThrust[iVar]->Write();
    HC[iVar]->Write();
    HD[iVar]->Write();
    HMjH[iVar]->Write();
    HBjWide[iVar]->Write();
    HBjTot[iVar]->Write();
    HnCharged[iVar]->Write();
    Hx[iVar]->Write();
    HxBSLD[iVar]->Write();
    HxBDelphi[iVar]->Write();
  }
  f.Close();

  cout << endl << "Enter q to terminate (will kill displays)..." << endl;
  char tmp;cin>>tmp;
  
  return 0;
}
