// VRdisplay.h is a part of the VINCIA plug-in to the PYTHIA 8 generator.
// Copyright (C) 2013 P. Skands
// VINCIA is licenced under the GNU GPL version 2.
// Please respect the MCnet Guidelines for Event Generator Authors and Users, 
//               http://www.montecarlonet.org/GUIDELINES

// This file contains header information for the Vincia ROOT utilities

#ifndef VINCIAROOTDISPLAY_H
#define VINCIAROOTDISPLAY_H

// Include the VINCIA ROOT headers 
// (ROOT and STDLIB headers included in VRsettings.h)

#include "VRfile.h"
#include "VRgraph.h"
#include "VRsettings.h"

// Put all the rest in Vincia namespace

namespace Vincia {
  
  // ------------------------------------------------------------------------
  // class VRdisplay
  // Contains the base class for a single 1D plot

  class VRdisplay {
    
  public:

    // Constructor
    VRdisplay(string displayNameIn, int xSizIn = 300, int ySizIn = 400,
              int xOffsetIn=-1, int yOffsetIn=-1) 
      : nPad(1), isInitPad(false), hasAxesSav(false), hasDataSav(false), 
      hasTitle(false), hasLegend(false), hasMCnames(false), hasDataRef(false),
      hasSignature(false) {
      displayNameSav   = displayNameIn;
      displayCanvasPtr = new TCanvas(displayNameIn.c_str(),
                                     displayNameIn.c_str(), 
                                     xOffsetIn, yOffsetIn,xSizIn,ySizIn);      
    }
  
    // Init pointers and set settings
    void init(VRsettings* settingsPtrIn);

    // Functions to Get    
    bool getLogY() { return logY; }
    bool getLinY() { return !logY; }
    bool getNormalize() { return normalize; }
    int  getOptStat() { return optStat; }
    double getSigmaMC() { return sigmaStat; }    
    double getNormFac() { return normFac; }
    string getName() { return displayNameSav; }

    // Functions to check whether axes and data have been defined
    bool hasAxes() {return hasAxesSav;}
    bool hasData() {return hasDataSav;}    

    // Add pad (for ratio to data or decomposition of vector of histos)
    void addPad();

    // Goto pad
    void gotoPad(int iPadIn = 1);

    // Functions to Set
    void setVerbose(int verboseIn) {verbose = verboseIn;};
    void setOptStat(int optStatIn) {optStat = optStatIn;}
    void setSigmaMC(double sigmaStatIn) {sigmaStat = sigmaStatIn;};
    void setNormalize(bool normalizeIn = true, double normFacIn = 1.0) {
      normalize  = normalizeIn;
      normFac    = normFacIn;
    }
    void setLogY(bool logYin = true) { 
      logY = logYin;
      displayCanvasPtr->cd(1); 
      if (logYin) gPad->SetLogy();
      else gPad->SetLogy(0);
      gPad->cd(1);
      if (logYin) gPad->SetLogy(); 
      else gPad->SetLogy(0);
    }
    void setLinY(bool linYin = true) { setLogY(!linYin); }
    void setSignature(string signatureIn="V I N C I A R O O T") {
      signature = signatureIn;
    }
    void setLeftLegend(bool leftLegendIn=true) { leftLegend = leftLegendIn; } 
    void setShowRatio(bool showRatioIn=true) { 
      showRatioPad = showRatioIn; 
    }
    void setShowRelUnc(bool showRelUncIn=true) {
      showRelUncPads = showRelUncIn;
    }

    // Check if axes and/or data has been defined
    void setAxes(TH1D* initHis);

    //------------------------------------------------------------------
    // VRdisplay: TH1D I/O

    // Add a histogram, not necessarily the first
    void addTH1DPtr(TH1D* hisPtrIn) {
      if (!hasAxes()) {
        setAxes(hisPtrIn);
        graphs.resize(0);
      }
      graphs.push_back(VRgraph(hisPtrIn));      
      // Default: assign color set n to graph number n
      graphs[graphs.size()-1].setColors(graphs.size()+graphsHis.size());
    }

    // Add a histogram (with vector uncertainties), not necessarily the first
    void addTH1DPtr(vector<TH1D*> hisPtrIn) {
      if (!hasAxes()) {
        setAxes(hisPtrIn[0]);
        graphs.resize(0);
      }
      graphs.push_back(VRgraph(hisPtrIn));
      graphs[graphs.size()-1].setColors(graphs.size()+graphsHis.size());
      
      // Add pad for decomposition of vector band
      if (hisPtrIn.size() >= 2 && showRelUncPads) addPad();
    }
    
    // Get histogram pointer corresponding to graph i, variation j
    TH1D* getTH1DPtr(int i=0, int j=0) {
      if ( i <= int(graphs.size())-1 ) return graphs[i].getTH1DPtr(j);
      else return NULL;
    }

    //========================================================================
    // VRdisplay: FILE I/O

    //------------------------------------------------------------------------
    // getFormat(): 
    // Function to convert a list of integers to vector<int> 
    void getFormat(vector<int>& format, int i1, int i2, int i3=-1, int i4=-1,
                   int i5=-1, int i6=-1, int i7=-1, int i8=-1, int i9=-1, 
                   int i10=-1, int i11=-1, int i12=-1) {
      // Store file format 
      format.push_back(i1);
      format.push_back(i2); 
      if (i3 >= 0) format.push_back(i3); else return;
      if (i4 >= 0) format.push_back(i4); else return;
      if (i5 >= 0) format.push_back(i5); else return;
      if (i6 >= 0) format.push_back(i6); else return;
      if (i7 >= 0) format.push_back(i7); else return;
      if (i8 >= 0) format.push_back(i8); else return;
      if (i9 >= 0) format.push_back(i9); else return;
      if (i10 >= 0) format.push_back(i10); else return;
      if (i11 >= 0) format.push_back(i11); else return;
      if (i12 >= 0) format.push_back(i12); else return;
    }

    //------------------------------------------------------------------------
    // setDataFile():
    // Set file to be treated as experimental data 
    bool setDataFile(string dataFileName, bool hasSysErrors = true) {
      if (hasSysErrors) return setDataFile(dataFileName,10,20,30,40,51,52,61,62);
      else return setDataFile(dataFileName,10,20,30,40,51,52);
    }
    bool setDataFile(string dataFileName, int i1, int i2, int i3=-1, int i4=-1, 
                     int i5=-1, int i6=-1, int i7=-1, int i8=-1, int i9=-1, 
                     int i10=-1, int i11=-1, int i12=-1);

    //------------------------------------------------------------------------
    // addHisFile() : 
    // Add file with saved histogram
    bool addHisFile(string hisFileName, bool hasSysErrors = false) {
      if (hasSysErrors) return addHisFile(hisFileName,10,20,30,40,51,52,61,62);
      else return addHisFile(hisFileName,10,20,30,40,51,52);
    }
    bool addHisFile(string hisFileName, int i1, int i2, int i3=-1, int i4=-1, 
                    int i5=-1, int i6=-1, int i7=-1, int i8=-1, int i9=-1, 
                    int i10=-1, int i11=-1, int i12=-1);

    //------------------------------------------------------------------------
    // Add various kinds of objects to display
    // (start with simple line. Can maybe be upgraded to general objects?)
    void addTLine(TLine* newLine) {
      linesPtr.push_back(newLine);
    }


    //------------------------------------------------------------------------
    // update(): 
    // Update Display 
    void update();

    //------------------------------------------------------------------------
    // saveDisplay():
    // Save display image to graphics file
    void saveDisplay(string prefix="none",  
                     string fileType="png",string option="Landscape");

    //------------------------------------------------------------------------
    // saveHistogram():
    // save main theory histogram to data file (in standard column format)
    // Note: extended format allows theory uncertainty variations to be included
    bool saveHistogram(string prefix="none", string ext="dat");

    //========================================================================
    // VRdisplay: Plot titles, legends, descriptions

    //------------------------------------------------------------------------
    // setTitle():
    // Set title
    void setTitle(); 

    //------------------------------------------------------------------------
    // setMCnames():
    // Set MC name title
    void setMCnames(string MCnamesIn);

    //------------------------------------------------------------------------
    // setDataRef():
    // Set Data reference title
    void setDataRef(string dataRefIn);

    //------------------------------------------------------------------------
    // setLegend():
    // Set Legend 
    // Optionally force alignment: align = -1(auto), 0(left), 1(right)
    void setLegend(vector<string> legendIn, int alignIn=-1); 

  private:

    //========================================================================
    // VRdisplay: private data members and functions
    
    // Canvas
    string displayNameSav;
    TCanvas* displayCanvasPtr;

    // Internal variables
    bool logY, normalize;
    bool showRatioPad, showRelUncPads;
    int verbose, optStat;
    double sigmaStat, normFac;

    // Input containers
    VRfile* dataFilePtr;
    vector<VRfile*> hisFilePtrs;
    
    // Global settings
    VRsettings* settingsPtr;

    // PADS
    int nPad;
    bool isInitPad;

    // PLOT

    // Use histogram to define axes
    TH1D* axesPtr;    
    TH1D* axesNormPtr;
    TH1D* axesVarPtr;
    TLine* xCtrPtr;
    TLine* xUppPtr;
    bool hasAxesSav;
    

    // MC Graphs 
    vector<VRgraph> graphs;
    vector<VRgraphNorm> graphsNorm;

    // His graphs (for comparison to saved data files)
    vector<VRgraphHis> graphsHis;
    vector<VRgraphNorm> graphsHisNorm;

    // RelUnc pad
    vector<VRgraphVar> graphsVar;

    // Data graph (and normlaized one- and two-sigma variants)
    VRgraphData* dataPtr;
    VRgraphData* dataTwoSigmaPtr;
    VRgraphNorm* dataNormPtr;
    VRgraphNorm* dataTwoSigmaNormPtr;
    bool hasDataSav;

    // Plot title 
    TLatex* titleTexPtr;
    bool hasTitle;

    // Objects
    vector<TLine*> linesPtr;
    
    // LEGEND

    // Main Legend 
    TLegend* legendPtr;
    bool hasLegend;
    bool leftLegend;
    string legendSav;

    // MC names and versions
    TLatex* MCnamesPtr;
    bool hasMCnames;
    string MCnamesSav;
    
    // Data reference
    TLatex* dataRefPtr;
    bool hasDataRef;

    // VinciaRoot Signature (on lower right-hand edge of plots)
    TLatex* signaturePtr;
    bool hasSignature;
    string signature;

  };

}

#endif
